@@ -45,7 +45,7 @@ class Espresso(FileIOCalculator):
             Generate a grid of k-points with this as the minimum distance,
             in A^-1 between them in reciprocal space. If set to None, kpts
             will be used instead.
-        kpts: (int, int, int) or dict
+        kpts: (int, int, int), dict, or BandPath
             If kpts is a tuple (or list) of 3 integers, it is interpreted
             as the dimensions of a Monkhorst-Pack grid.
             If kpts is a dict, it will either be interpreted as a path
@@ -64,8 +64,8 @@ class Espresso(FileIOCalculator):
 
         .. note::
            Band structure plots can be made as follows:
-           
-           
+
+
            1. Perform a regular self-consistent calculation,
               saving the wave functions at the end, as well as
               getting the Fermi energy:
@@ -75,26 +75,24 @@ class Espresso(FileIOCalculator):
               >>> atoms.set_calculator(calc)
               >>> atoms.get_potential_energy()
               >>> fermi_level = calc.get_fermi_level()
-              
+
            2. Perform a non-self-consistent 'band structure' run
               after updating your input_data and kpts keywords:
-              
+
               >>> input_data['control'].update({'calculation':'bands',
               >>>                               'restart_mode':'restart',
               >>>                               'verbosity':'high'})
               >>> calc.set(kpts={<your Brillouin zone path>},
               >>>          input_data=input_data)
               >>> calc.calculate(atoms)
-              
+
            3. Make the plot using the BandStructure functionality,
               after setting the Fermi level to that of the prior
               self-consistent calculation:
-              
+
               >>> bs = calc.band_structure()
               >>> bs.reference = fermi_energy
               >>> bs.plot()
-                  
-           
 
         """
         FileIOCalculator.__init__(self, restart, ignore_bad_restart_file,
@@ -143,8 +141,3 @@ class Espresso(FileIOCalculator):
         if nspins is None:
             warnings.warn(warn_template % 'Number of spins')
         return nspins
-
-    def socket_driver(self, **kwargs):
-        from ase.calculators.socketio import SocketIOCalculator
-        calc = SocketIOCalculator(self, **kwargs)
-        return calc
