@@ -40,6 +40,7 @@ class GUI(View, Status):
             images = Images(images)
 
         self.images = images
+        self.observers = []
 
         self.config = read_defaults()
         if show_bonds:
@@ -331,7 +332,17 @@ class GUI(View, Status):
 
     def quick_info_window(self, key=None):
         from ase.gui.quickinfo import info
-        ui.Window(_('Quick Info')).add(info(self))
+        info_win = ui.Window(_('Quick Info'))
+        info_win.add(info(self))
+
+        # Update quickinfo window when we change frame
+        def update(window):
+            exists = window.exists
+            if exists:
+                # Only update if we exist
+                window.things[0].text = info(self)
+            return exists
+        self.attach(update, info_win)
 
     def bulk_window(self):
         SetupBulkCrystal(self)
@@ -517,6 +528,14 @@ class GUI(View, Status):
                                      'ase/ase/gui/gui.html')),
               M(_('Webpage ...'), webpage)])]
 
+    def attach(self, function, *args, **kwargs):
+        self.observers.append((function, args, kwargs))
+
+    def call_observers(self):
+        # Use function return value to determine if we keep observer
+        self.observers = [(function, args, kwargs) for (function, args, kwargs)
+                          in self.observers if function(*args, **kwargs)]
+
     def repeat_poll(self, callback, ms, ensure_update=True):
         """Invoke callback(gui=self) every ms milliseconds.
 
