@@ -25,6 +25,8 @@ from ase.calculators.singlepoint import SinglePointCalculator
 from ase.spacegroup.spacegroup import Spacegroup
 from ase.parallel import paropen
 from ase.utils import basestring
+from ase.constraints import FixAtoms, FixCartesian
+from ase.io.formats import index2range
 
 __all__ = ['read_xyz', 'write_xyz', 'iread_xyz']
 
@@ -36,10 +38,10 @@ PROPERTY_NAME_MAP = {'positions': 'pos',
 REV_PROPERTY_NAME_MAP = dict(zip(PROPERTY_NAME_MAP.values(),
                                  PROPERTY_NAME_MAP.keys()))
 
-KEY_QUOTED_VALUE = re.compile(r'([A-Za-z_]+[A-Za-z0-9_-]*)' +
-                              r'\s*=\s*["\{\}]([^"\{\}]+)["\{\}]\s*')
-KEY_VALUE = re.compile(r'([A-Za-z_]+[A-Za-z0-9_]*)\s*=' +
-                       r'\s*([^\s]+)\s*')
+KEY_QUOTED_VALUE = re.compile(r'([A-Za-z_]+[A-Za-z0-9_-]*)'
+                              + r'\s*=\s*["\{\}]([^"\{\}]+)["\{\}]\s*')
+KEY_VALUE = re.compile(r'([A-Za-z_]+[A-Za-z0-9_]*)\s*='
+                       + r'\s*([^\s]+)\s*')
 KEY_RE = re.compile(r'([A-Za-z_]+[A-Za-z0-9_-]*)\s*')
 
 UNPROCESSED_KEYS = ['uid']
@@ -222,7 +224,7 @@ def key_val_str_to_dict_regex(s):
     return d
 
 
-def key_val_dict_to_str(d, sep=' '):
+def key_val_dict_to_str(d, sep=' ', tolerant=False):
     """
     Convert atoms.info dictionary to extended XYZ string representation
     """
@@ -240,9 +242,23 @@ def key_val_dict_to_str(d, sep=' '):
         if isinstance(val, dict):
             continue
         if hasattr(val, '__iter__'):
-            val = np.array(val)
-            val = ' '.join(str(type_val_map.get((type(x), x), x))
-                           for x in val.reshape(val.size, order='F'))
+            try:
+                val = np.array(list(val))
+                val = ' '.join(str(type_val_map.get((type(x), x), x))
+                               for x in val.reshape(val.size, order='F'))
+            except (TypeError, ValueError) as exc:
+                # It may fail if the type is unhashable
+                # ValueError is only relevant for non-uniform arrays
+                # with older numpy (1.10.4) and affects the creation
+                # of the array.  Can be removed in the future.
+                if tolerant:
+                    warnings.warn('Skipping unhashable information '
+                                  '{0}'.format(key))
+                    continue
+                else:
+                    raise RuntimeError('Unhashable object in info dictionary,'
+                                       ' please remove it or use '
+                                       'tolerant=True') from exc
             val.replace('[', '')
             val.replace(']', '')
         elif isinstance(val, Spacegroup):
@@ -329,7 +345,7 @@ def _read_xyz_frame(lines, natoms, properties_parser=key_val_str_to_dict, nvec=0
         # is True in all directions
         pbc = [True, True, True]
     elif nvec > 0:
-        #cell information given as pseudo-Atoms
+        # cell information given as pseudo-Atoms
         pbc = [False, False, False]
 
     cell = None
@@ -338,8 +354,8 @@ def _read_xyz_frame(lines, natoms, properties_parser=key_val_str_to_dict, nvec=0
         cell = info['Lattice'].T
         del info['Lattice']
     elif nvec > 0:
-        #cell information given as pseudo-Atoms
-        cell = np.zeros((3,3))
+        # cell information given as pseudo-Atoms
+        cell = np.zeros((3, 3))
 
     if 'Properties' not in info:
         # Default set of properties is atomic symbols and positions only
@@ -364,7 +380,7 @@ def _read_xyz_frame(lines, natoms, properties_parser=key_val_str_to_dict, nvec=0
         raise XYZError('Badly formatted data '
                        'or end of file reached before end of frame')
 
-    #Read VEC entries if present
+    # Read VEC entries if present
     if nvec > 0:
         for ln in range(nvec):
             try:
@@ -380,9 +396,10 @@ def _read_xyz_frame(lines, natoms, properties_parser=key_val_str_to_dict, nvec=0
                 n = int(entry[0][3:])
                 if n != ln + 1:
                     raise XYZError('Expected VEC{}, got VEC{}'
-                                   .format(ln+1,n))
+                                   .format(ln + 1, n))
             except:
-                raise XYZError('Expected VEC{}, got VEC{}'.format(ln+1,entry[0][3:]))
+                raise XYZError('Expected VEC{}, got VEC{}'.format(
+                    ln + 1, entry[0][3:]))
 
             cell[ln] = np.array([float(x) for x in entry[1:]])
             pbc[ln] = True
@@ -397,7 +414,7 @@ def _read_xyz_frame(lines, natoms, properties_parser=key_val_str_to_dict, nvec=0
             value = data[name]
         else:
             value = np.vstack([data[name + str(c)]
-                              for c in range(cols)]).T
+                               for c in range(cols)]).T
         arrays[ase_name] = value
 
     symbols = None
@@ -427,11 +444,22 @@ def _read_xyz_frame(lines, natoms, properties_parser=key_val_str_to_dict, nvec=0
     atoms = Atoms(symbols=symbols,
                   positions=positions,
                   numbers=numbers,
-                  charges = charges,
+                  charges=charges,
                   cell=cell,
                   pbc=pbc,
                   info=info)
 
+    # Read and set constraints
+    if 'move_mask' in arrays:
+        if properties['move_mask'][1] == 3:
+            atoms.set_constraint(
+                [FixCartesian(a, mask=arrays['move_mask'][a, :]) for a in range(natoms)])
+        elif properties['move_mask'][1] == 1:
+            atoms.set_constraint(FixAtoms(mask=~arrays['move_mask']))
+        else:
+            raise XYZError('Not implemented constraint')
+        del arrays['move_mask']
+
     for name, array in arrays.items():
         atoms.new_array(name, array)
 
@@ -493,6 +521,7 @@ def ixyzchunks(fd):
 
 class ImageIterator:
     """"""
+
     def __init__(self, ichunks):
         self.ichunks = ichunks
 
@@ -566,7 +595,7 @@ def read_xyz(fileobj, index=-1, properties_parser=key_val_str_to_dict):
         fileobj.readline()  # read comment line
         for i in range(natoms):
             fileobj.readline()
-        #check for VEC
+        # check for VEC
         nvec = 0
         while True:
             lastPos = fileobj.tell()
@@ -582,33 +611,7 @@ def read_xyz(fileobj, index=-1, properties_parser=key_val_str_to_dict):
         if last_frame is not None and len(frames) > last_frame:
             break
 
-    if isinstance(index, int):
-        if index < 0:
-            tmpsnp = len(frames) + index
-            trbl = range(tmpsnp, tmpsnp + 1, 1)
-        else:
-            trbl = range(index, index + 1, 1)
-    elif isinstance(index, slice):
-        start = index.start
-        stop = index.stop
-        step = index.step
-
-        if start is None:
-            start = 0
-        elif start < 0:
-            start = len(frames) + start
-
-        if step is None:
-            step = 1
-
-        if stop is None:
-            stop = len(frames)
-        elif stop < 0:
-            stop = len(frames) + stop
-
-        trbl = range(start, stop, step)
-        if step < 0:
-            trbl.reverse()
+    trbl = index2range(index, len(frames))
 
     for index in trbl:
         frame_pos, natoms, nvec = frames[index]
@@ -619,7 +622,7 @@ def read_xyz(fileobj, index=-1, properties_parser=key_val_str_to_dict):
 
 
 def output_column_format(atoms, columns, arrays,
-                         write_info=True, results=None):
+                         write_info=True, results=None, tolerant=False):
     """
     Helper function to build extended XYZ comment line
     """
@@ -633,8 +636,8 @@ def output_column_format(atoms, columns, arrays,
 
     # NB: Lattice is stored as tranpose of ASE cell,
     # with Fortran array ordering
-    lattice_str = ('Lattice="' +
-                   ' '.join([str(x) for x in np.reshape(atoms.cell.T,
+    lattice_str = ('Lattice="'
+                   + ' '.join([str(x) for x in np.reshape(atoms.cell.T,
                                                         9, order='F')]) +
                    '"')
 
@@ -653,8 +656,8 @@ def output_column_format(atoms, columns, arrays,
         property_names.append(property_name)
         property_types.append(property_type)
 
-        if (len(array.shape) == 1 or
-                (len(array.shape) == 2 and array.shape[1] == 1)):
+        if (len(array.shape) == 1
+                or (len(array.shape) == 2 and array.shape[1] == 1)):
             ncol = 1
             dtypes.append((column, dtype))
         else:
@@ -681,7 +684,7 @@ def output_column_format(atoms, columns, arrays,
     if results is not None:
         info.update(results)
     info['pbc'] = atoms.get_pbc()  # always save periodic boundary conditions
-    comment_str += ' ' + key_val_dict_to_str(info)
+    comment_str += ' ' + key_val_dict_to_str(info, tolerant=tolerant)
 
     dtype = np.dtype(dtypes)
     fmt = ''.join(formats) + '\n'
@@ -690,7 +693,8 @@ def output_column_format(atoms, columns, arrays,
 
 
 def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
-              write_results=True, plain=False, vec_cell=False, append=False):
+              write_results=True, plain=False, vec_cell=False, append=False, 
+              tolerant=False):
     """
     Write output in extended XYZ format
 
@@ -717,10 +721,10 @@ def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
             fr_cols = columns[:]
 
         if fr_cols is None:
-            fr_cols = (['symbols', 'positions'] +
-                       [key for key in atoms.arrays.keys() if
+            fr_cols = (['symbols', 'positions']
+                       + [key for key in atoms.arrays.keys() if
                         key not in ['symbols', 'positions', 'numbers',
-                                    'species', 'pos']])
+                                      'species', 'pos']])
 
         if vec_cell:
             plain = True
@@ -734,15 +738,15 @@ def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
         per_atom_results = {}
         if write_results:
             calculator = atoms.get_calculator()
-            if (calculator is not None and
-                    isinstance(calculator, Calculator)):
+            if (calculator is not None
+                    and isinstance(calculator, Calculator)):
                 for key in all_properties:
                     value = calculator.results.get(key, None)
                     if value is None:
                         # skip missing calculator results
                         continue
-                    if (isinstance(value, np.ndarray) and
-                            value.shape[0] == len(atoms)):
+                    if (isinstance(value, np.ndarray)
+                            and value.shape[0] == len(atoms)):
                         # per-atom quantities (forces, energies, stresses)
                         per_atom_results[key] = value
                     else:
@@ -779,24 +783,40 @@ def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
         if pos.shape != (natoms, 3) or pos.dtype.kind != 'f':
             raise ValueError('Second column must be position-like')
 
-        #if vec_cell add cell information as pseudo-atoms
+        # if vec_cell add cell information as pseudo-atoms
         if vec_cell:
             pbc = list(atoms.get_pbc())
             cell = atoms.get_cell()
 
             if True in pbc:
                 nPBC = 0
-                for i,b in enumerate(pbc):
+                for i, b in enumerate(pbc):
                     if b:
                         nPBC += 1
-                        symbols.append('VEC'+str(nPBC))
+                        symbols.append('VEC' + str(nPBC))
                         pos = np.vstack((pos, cell[i]))
-                #add to natoms
+                # add to natoms
                 natoms += nPBC
                 if pos.shape != (natoms, 3) or pos.dtype.kind != 'f':
-                    raise ValueError('Pseudo Atoms containing cell have bad coords')
-
+                    raise ValueError(
+                        'Pseudo Atoms containing cell have bad coords')
 
+        # Move mask
+        if 'move_mask' in fr_cols:
+            cnstr = images[0]._get_constraints()
+            if len(cnstr) > 0:
+                c0 = cnstr[0]
+                if isinstance(c0, FixAtoms):
+                    cnstr = np.ones((natoms,), dtype=np.bool)
+                    for idx in c0.index:
+                        cnstr[idx] = False
+                elif isinstance(c0, FixCartesian):
+                    for i in range(len(cnstr)):
+                        idx = cnstr[i].a
+                        cnstr[idx] = cnstr[i].mask
+                    cnstr = np.asarray(cnstr)
+            else:
+                fr_cols.remove('move_mask')
 
         # Collect data to be written out
         arrays = {}
@@ -807,6 +827,8 @@ def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
                 arrays[column] = atoms.arrays[column]
             elif column == 'symbols':
                 arrays[column] = np.array(symbols)
+            elif column == 'move_mask':
+                arrays[column] = cnstr
             else:
                 raise ValueError('Missing array "%s"' % column)
 
@@ -824,7 +846,9 @@ def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
                                                        fr_cols,
                                                        arrays,
                                                        write_info,
-                                                       per_frame_results)
+                                                       per_frame_results, 
+                                                       tolerant)
+
         if plain or comment != '':
             # override key/value pairs with user-speficied comment string
             comm = comment
@@ -840,7 +864,8 @@ def write_xyz(fileobj, images, comment='', columns=None, write_info=True,
                     data[column + str(c)] = value[:, c]
 
         nat = natoms
-        if vec_cell: nat -= nPBC
+        if vec_cell:
+            nat -= nPBC
         # Write the output
         fileobj.write('%d\n' % nat)
         fileobj.write('%s\n' % comm)
