@@ -2,12 +2,23 @@ import datetime
 import json
 
 import numpy as np
-from ase.utils import basestring
+from ase.utils import reader, writer
 
 
 class MyEncoder(json.JSONEncoder):
     def default(self, obj):
-        if isinstance(obj, np.ndarray):
+        if hasattr(obj, 'todict'):
+            d = obj.todict()
+
+            if not isinstance(d, dict):
+                raise RuntimeError('todict() of {} returned object of type {} '
+                                   'but should have returned dict'
+                                   .format(obj, type(d)))
+            if hasattr(obj, 'ase_objtype'):
+                d['__ase_objtype__'] = obj.ase_objtype
+
+            return d
+        if isinstance(obj, np.ndarray) or hasattr(obj, '__array__'):
             if obj.dtype == complex:
                 return {'__complex_ndarray__': (obj.real.tolist(),
                                                 obj.imag.tolist())}
@@ -18,8 +29,6 @@ class MyEncoder(json.JSONEncoder):
             return bool(obj)
         if isinstance(obj, datetime.datetime):
             return {'__datetime__': obj.isoformat()}
-        if hasattr(obj, 'todict'):
-            return obj.todict()
         return json.JSONEncoder.default(self, obj)
 
 
@@ -33,6 +42,30 @@ def object_hook(dct):
     if '__complex_ndarray__' in dct:
         r, i = (np.array(x) for x in dct['__complex_ndarray__'])
         return r + i * 1j
+
+    if '__ase_objtype__' in dct:
+        objtype = dct.pop('__ase_objtype__')
+        dct = numpyfy(dct)
+
+        # We just try each object type one after another and instantiate
+        # them manually, depending on which kind it is.
+        # We can formalize this later if it ever becomes necessary.
+        if objtype == 'cell':
+            from ase.cell import Cell
+            obj = Cell(**dct)
+        elif objtype == 'bandstructure':
+            from ase.dft.band_structure import BandStructure
+            obj = BandStructure(**dct)
+        elif objtype == 'bandpath':
+            from ase.dft.kpoints import BandPath
+            obj = BandPath(path=dct.pop('labelseq'), **dct)
+        else:
+            raise RuntimeError('Do not know how to decode object type {} '
+                               'into an actual object'.format(objtype))
+
+        assert obj.ase_objtype == objtype
+        return obj
+
     return dct
 
 
@@ -69,11 +102,12 @@ def decode(txt):
     return numpyfy(mydecode(txt))
 
 
-def read_json(name):
-    if isinstance(name, basestring):
-        fd = open(name, 'r')
-    else:
-        fd = name
+@reader
+def read_json(fd):
     dct = decode(fd.read())
-    fd.close()
     return dct
+
+
+@writer
+def write_json(fd, obj):
+    fd.write(encode(obj))
