@@ -7,6 +7,7 @@ which ASE is installed and how - but assumes that it is run from the
 ASE root directory."""
 
 import os
+os.environ['LANGUAGE'] = 'C'
 import subprocess
 import re
 import argparse
@@ -34,9 +35,6 @@ bash = runcmd
 def py(cmd, output=False):
     return runcmd('python3 {}'.format(cmd))
 
-def py2(cmd, output=False):
-    return runcmd('python2 {}'.format(cmd))
-
 def git(cmd, error_ok=False):
     cmd = 'git {}'.format(cmd)
     return runcmd(cmd, output=True, error_ok=error_ok)
@@ -215,7 +213,6 @@ News
     git('tag -s {0} -m "ase-{0}"'.format(version))
 
     py('setup.py sdist > setup_sdist.log')
-    py2('setup.py bdist_wheel > setup_bdist_wheel2.log')
     py('setup.py bdist_wheel > setup_bdist_wheel3.log')
     bash('gpg --armor --yes --detach-sign dist/ase-{}.tar.gz'.format(version))
     git('checkout -b web-page')
@@ -241,7 +238,6 @@ News
     print('git merge {}'.format(branchname))
     print('twine upload '
           'dist/ase-{v}.tar.gz '
-          'dist/ase-{v}-py2-none-any.whl '
           'dist/ase-{v}-py3-none-any.whl '
           'dist/ase-{v}.tar.gz.asc'.format(v=version))
     print('git push --tags origin master  # Assuming your remote is "origin"')
